//-----------------------------------------------------------------------------
//  TimpyH_MSD_Sect_Serv.c
//-----------------------------------------------------------------------------
//
//  Copyright 2006, 2007 Chiaki Nakajima
//  Original copyright 2006 Silicon Laboratories, Inc.
//
//  Release 1.1
//      E\[X̐
//      EgpvZbTF340ɌC"__F340_VER__"`QƂ폜
//      Et@C"F34x_""TimpyH_"ɕύX
//
//  Release 1.0
//      ESiLabsUSB-MSD-RD(Release 1.1)ɁCTimpy Rev5.0pɑSʓIɕύX
//      ESDCCֈڐA
//      Esvȋ@\폜
//
//-----------------------------------------------------------------------------
//
//  Copyright 2006 Silicon Laboratories, Inc.
//  http://www.silabs.com
//
//  Release 1.1
//    -All changes by PKC
//    -09 JUN 2006
//    -No changes; incremented revision number to match project revision
//
//  Release 1.0
//    -Initial Release
//
//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------

#include "TimpyH_Common_Defs.h"
#include "TimpyH_MSD_Sect_Serv.h"
#include "TimpyH_MSD_MMC.h"

#define HIDDEN_SECTORS  0x00                    // MMC-specific
#define DIRENTRY_SIZE   0x20                    // Size of an entry in the root directory

BYTE __xdata Scratch[PHYSICAL_BLOCK_SIZE];      // Buffer for read/write transfers
bootrecord_small __xdata MBR;                   // Permanent copy of important fields from bootrecord

extern xdata unsigned long PHYSICAL_BLOCKS;

//-----------------------------------------------------------------------------
//  Functions
//-----------------------------------------------------------------------------

extern void print_scratch();

//----------------------------------------------------------------------------
//  Sect_Validate
//----------------------------------------------------------------------------
//
//  Function checks the validate of bootrecord.
//
//  Parameters   :
//  Return Value :
//----------------------------------------------------------------------------

static BYTE Sect_Validate_Nest_Level = 0;

void Sect_Validate(void) reentrant
{
    UINT fat_sec = 0;
    bootrecord_large *bootrecord = Scratch;

    MBR.valid = 0;
    MBR.hidden_sectors = 0;

    if ((bootrecord->signature[0] != 0x55) || (bootrecord->signature[1] != 0xAA)) {
        return;
    }
    if (PHYSICAL_BLOCK_SIZE != /*ntohs*/(bootrecord->bytes_per_sector)) {
        goto Check_MBR;
    }
    if (bootrecord->filesystem[0] != 'F'
     || bootrecord->filesystem[1] != 'A'
     || bootrecord->filesystem[2] != 'T'
     || bootrecord->filesystem[3] != '1'
     || bootrecord->filesystem[4] != '6') {
        goto Check_MBR;
    }

    // Make a permanent copy of the important fields of the bootrecord:
    MBR.fat_copies = bootrecord->fat_copies;
    MBR.root_directory_entries = /*ntohs*/(bootrecord->root_directory_entries);
    MBR.number_of_sectors = /*ntohs*/(bootrecord->number_of_sectors);
    MBR.sectors_per_fat = /*ntohs*/(bootrecord->sectors_per_fat);
    MBR.total_sectors = /*ntohl*/(bootrecord->total_sectors);
    MBR.reserved_sectors = /*ntohs*/(bootrecord->reserved_sectors);
    MBR.sectors_per_cluster = bootrecord->sectors_per_cluster;
    MBR.valid = 1;
    return;
    
    Check_MBR: {
        if (Sect_Validate_Nest_Level > 1) return;
        Sect_Validate_Nest_Level++;
        if ((Scratch[0x1Be] == 0x80) || (Scratch[0x1Be] == 0x00)) {     // checks if this sector is not a MBR
                                                                        // partition is active

            fat_sec = *(UINT *) &Scratch[0x1c6];
//          fat_sec = ntohs(fat_sec);
            Sect_Read(fat_sec);
            Sect_Validate();
            MBR.hidden_sectors = fat_sec;
        }
    }
}

//----------------------------------------------------------------------------
//  Sect_Init
//----------------------------------------------------------------------------
//
//  Function initializes memory card (Compact Flash or MMC), reads sector 0 and 
//  checks the validate of bootrecord.
//
//  Parameters   :
//  Return Value :
//----------------------------------------------------------------------------

void Sect_Init(void)
{
    MMC_FLASH_Init(); 
    Sect_Read(0);
    Sect_Validate_Nest_Level = 0;
    Sect_Validate();
}

//----------------------------------------------------------------------------
//  Sect_Sectors
//----------------------------------------------------------------------------
//
//  Returns number of sectors.
//
//  Parameters   :
//  Return Value :
//----------------------------------------------------------------------------

ULONG Sect_Sectors(void)
{
    return PHYSICAL_BLOCKS;                     // MBR.number_of_sectors;
}

//----------------------------------------------------------------------------
//  Sect_Read
//----------------------------------------------------------------------------
//
//  Reads one sector into Scratch buffer
//
//  Parameters   : sector - sector's number
//  Return Value : error number
//----------------------------------------------------------------------------

UINT Sect_Read(ULONG sector) 
{
    UINT /*xdata*/ error = 0;
    BYTE /*xdata*/ loopguard = 0;
    
    while ((error = MMC_FLASH_Block_Read(sector + HIDDEN_SECTORS, Scratch)) != 0) {
        if(!++loopguard) break;
    }
    
    return error;
}

//----------------------------------------------------------------------------
//  Sect_Write
//----------------------------------------------------------------------------
//
//  It writes one sector from Scratch buffer
//
//  Parameters   : sector - sector's number
//  Return Value :
//----------------------------------------------------------------------------

void Sect_Write(ULONG sector) 
{
    MMC_FLASH_Block_Write(sector + HIDDEN_SECTORS, Scratch);    // wozb - 27-09-2005 instead of this write data from uart

    if (sector == 0) {                          // After reformatting by the PC, we must re-init the sector server.
        Sect_Validate_Nest_Level = 0;
        Sect_Validate();
    }
}

//----------------------------------------------------------------------------
//  Sect_Write_Multi_Fat
//----------------------------------------------------------------------------
//
//  Automatically handle multiple FAT copies
//
//  Parameters   : sector - sector's number
//  Return Value :
//----------------------------------------------------------------------------

void Sect_Write_Multi_Fat(ULONG sector)
{
    if ((sector < Sect_Fat1()) || (sector >= Sect_Root_Dir())) {
                                                // This is a 'normal' block, not in the FAT:
        Sect_Write(sector);
    } else {
                                                // Writing to one of the FAT's will automagically write the same block to the 
                                                // other FAT copies.
        while (sector >= Sect_Fat2()) {         // Decrement 'sector' to refer to 1st FAT copy.
            sector -= MBR.sectors_per_fat;
        }
        while (sector < Sect_Root_Dir()) {      // Write same data to each FAT copy.
            Sect_Write(sector);
            sector += MBR.sectors_per_fat;
        }
    }
}

//----------------------------------------------------------------------------
//  Sect_Root_Dir
//----------------------------------------------------------------------------
//
//  Returns number of sector for root directory
//
//  Parameters   :
//  Return Value : number of sectors
//----------------------------------------------------------------------------

UINT Sect_Root_Dir(void) 
{
    return MBR.hidden_sectors
         + MBR.reserved_sectors                 /* Boot record followed by FATs */ 
         + (MBR.fat_copies * MBR.sectors_per_fat);
}

//----------------------------------------------------------------------------
//  Sect_Root_Dir_Last
//----------------------------------------------------------------------------
//
//  Returns number of last sector for root directory
//
//  Parameters   :
//  Return Value : number of last sector
//----------------------------------------------------------------------------

UINT Sect_Root_Dir_Last(void)
{
    return Sect_Root_Dir() - 1 + (MBR.root_directory_entries * DIRENTRY_SIZE) / PHYSICAL_BLOCK_SIZE;
}

//----------------------------------------------------------------------------
//  Sect_File_Data
//----------------------------------------------------------------------------
//
//  Returns first sector of file data
//
//  Parameters   :
//  Return Value : number of sector
//----------------------------------------------------------------------------

UINT Sect_File_Data(void)
{
    return Sect_Root_Dir_Last() + 1 - (MBR.sectors_per_cluster * 2);    // First file data block is called "number 2".
}

//----------------------------------------------------------------------------
//  Sect_Fat1
//----------------------------------------------------------------------------
//
//  Returns first sector of 1-st FAT
//
//  Parameters   :
//  Return Value : number of sector
//----------------------------------------------------------------------------

UINT Sect_Fat1(void)
{
    return MBR.hidden_sectors + MBR.reserved_sectors;
}

//----------------------------------------------------------------------------
//  Sect_Fat2
//----------------------------------------------------------------------------
//
//  Returns first sector of 2-st FAT
//
//  Parameters   :
//  Return Value : number of sector
//----------------------------------------------------------------------------

UINT Sect_Fat2(void)
{
    return MBR.hidden_sectors + MBR.reserved_sectors + MBR.sectors_per_fat;
}

/* End of TimpyH_MSD_Sect_Serv.c */
